{-|
Module      : Test.API.Notifications
Description : Tests for the notification mechanism (central exchange, dispatcher, websockets)
Copyright   : (c) CNRS, 2017-Present
License     : AGPL + CECILL v3
Maintainer  : team@gargantext.org
Stability   : experimental
Portability : POSIX

-}

{-# OPTIONS_GHC -Wno-orphans  #-}

    
module Test.API.Notifications (
    tests
  ) where

import Control.Concurrent (forkIO, killThread, threadDelay)
import Control.Concurrent.STM.TChan
import Control.Monad.STM (atomically)
import Data.Aeson qualified as Aeson
import Data.Maybe (isJust)
import Gargantext.Core.Notifications.CentralExchange qualified as CE
import Gargantext.Core.Notifications.CentralExchange.Types qualified as CET
import Gargantext.Core.Notifications.Dispatcher qualified as D
import Gargantext.Core.Notifications.Dispatcher.Types qualified as DT
import Gargantext.Core.Config.Types (NotificationsConfig(..))
import Network.WebSockets.Client qualified as WS
import Network.WebSockets.Connection qualified as WS
import Prelude
import Test.API.Setup (withTestDBAndNotifications)  -- , setupEnvironment, createAliceAndBob)
import Test.Hspec
import Test.Instances ()


tests :: NotificationsConfig -> D.Dispatcher -> Spec
tests nc dispatcher = sequential $ aroundAll (withTestDBAndNotifications dispatcher) $ do
  describe "Dispatcher, Central Exchange, WebSockets" $ do
    it "simple WS notification works" $ \((_testEnv, port), _) -> do
      let topic = DT.UpdateTree 0
      tchan <- newTChanIO :: IO (TChan (Maybe DT.Notification))
      -- setup a websocket connection
      let wsConnect = do
            WS.runClient "127.0.0.1" port "/ws" $ \conn -> do
              -- We wait a bit before the server settles
              threadDelay (100 * millisecond)
              WS.sendTextData conn $ Aeson.encode (DT.WSSubscribe topic)
              d <- WS.receiveData conn
              let dec = Aeson.decode d :: Maybe DT.Notification
              atomically $ writeTChan tchan dec
              -- atomically $ TVar.writeTVar tvar (Aeson.decode d)
            putStrLn "[WSClient] after"
            
      -- wait a bit to settle
      threadDelay (100 * millisecond)
    
      wsConnection <- forkIO $ wsConnect
      
      -- wait a bit to connect
      threadDelay (100 * millisecond)
      threadDelay (500 * millisecond)

      CE.notify nc $ CET.UpdateTreeFirstLevel 0

      -- d <- TVar.readTVarIO tvar
      md <- atomically $ readTChan tchan
      killThread wsConnection

      md `shouldSatisfy` isJust
      let (Just (DT.Notification topic' message')) = md
      topic' `shouldBe` topic
      message' `shouldBe` DT.MEmpty


millisecond :: Int
millisecond = 1000
